#!/bin/bash
USE_SCORING=0
. ../../..//testdata_tools/gen.sh

# ulimit -s unlimited

use_solution nils.cpp opt

compile gen_complete.py
compile gen_complete_bip.py
compile gen_cycle.py
compile gen_cycle_conjoined.py
compile gen_isolated.py
compile gen_rand.py
compile gen_girth.cpp
compile gen_girth_reg.cpp

MAXN=3000
MAXE=6000

sample 1
sample 2

# Complete graphs
tc k110 gen_complete N=110
tc k110_rem gen_complete N=110 remove=10

# Bipartite complete graphs
tc k77_77 gen_complete_bip N=77
tc k77_77_rm gen_complete_bip N=77 remove=10
tc k77_77_rm_with_c5 gen_complete_bip N=77 remove=10 add_c5=yes

# Different kinds of cycles
tc cyc3000 gen_cycle N=$MAXN
tc cyc1500-disjoint gen_cycle N=1500,1500
tc cyc2999 gen_cycle N=2999
tc cyc1499-disjoint gen_cycle N=1499,1499
tc cyc1499_1500-disjoint gen_cycle N=1499,1500
tc cyc1500_1499-disjoint gen_cycle N=1500,1499
tc cyc1500_1499-conjoined gen_cycle N=1500,1499 conjoined=yes
tc cyc1500_1500-conjoined gen_cycle N=1500,1500 conjoined=yes
tc cyc1499_1499-conjoined gen_cycle N=1499,1499 conjoined=yes

# Isolated vertices
tc isolated gen_isolated N=$MAXN

# Random graphs; almost always have triangles
tc rand-113 gen_rand N=113 E=$MAXE # 95% of edges
tc rand-241 gen_rand N=241 E=$MAXE # 20% of edges
tc rand-629 gen_rand N=629 E=$MAXE # 3% of edges
tc rand-1100 gen_rand N=1100 E=$MAXE # 1% of edges
tc rand-2420 gen_rand N=2420 E=$MAXE # 0.2% of edges
tc rand-3000 gen_rand N=3000 E=$MAXE # 0.15% of edges
tc rand-3000-5000 gen_rand N=3000 E=5000
tc rand-3000-3000 gen_rand N=3000 E=3000

# Random graphs with minimum girth
tc girth-300-1000-6 gen_girth N=300 E=1000 G=6

tc girth-3000-3000-4 gen_girth N=3000 E=3000 G=4
tc girth-3000-3000-5 gen_girth N=3000 E=3000 G=5
tc girth-3000-3000-74 gen_girth N=3000 E=3000 G=74
tc girth-3000-3000-75 gen_girth N=3000 E=3000 G=75
tc girth-3000-3000-151 gen_girth N=3000 E=3000 G=151
tc girth-3000-3000-150 gen_girth N=3000 E=3000 G=150

tc girth-3000-4000-15 gen_girth N=3000 E=3000 G=15
tc girth-3000-4000-16 gen_girth N=3000 E=3000 G=16

tc girth-3000-6000-4 gen_girth N=3000 E=6000 G=4
tc girth-3000-6000-5 gen_girth N=3000 E=6000 G=5
tc girth-3000-6000-9 gen_girth N=3000 E=6000 G=9
tc girth-3000-6000-10 gen_girth N=3000 E=6000 G=10

# Regular graphs with minimum girth
tc reg-3000-3 gen_girth_reg N=3000 K=3 c=1
tc reg-3000-4 gen_girth_reg N=3000 K=4 c=1
tc reg-2000-3 gen_girth_reg N=2000 K=3 c=1
tc reg-1000-4 gen_girth_reg N=1000 K=4 c=1
tc reg-1000-6 gen_girth_reg N=1000 K=6 c=1
