#!/usr/bin/python3

import sys
import random

default = {
    "n": 10,
    "m": 10,
    "mode": "random",
}

def cmdlinearg(name):
    for arg in sys.argv:
        if arg.startswith(name + "="):
            return arg.split("=")[1]
    return default[name]

def main():

    random.seed(int(sys.argv[-1]))
    n = int(cmdlinearg("n"))
    m = int(cmdlinearg("m"))
    mode = cmdlinearg("mode")
    
    edge_set = set()
    edges = []

    if mode == "random":
        for _ in range(m):
            a = random.randrange(0, n)
            b = random.randrange(0, n)
            while a == b or (a, b) in edge_set:
                a = random.randrange(0, n)
                b = random.randrange(0, n)
            edge_set.add((a, b))
            edge_set.add((b, a))
            edges.append((a, b))
    elif mode == "stars":
        tot = list(range(n))
        random.shuffle(tot)
        x = 0
        while m > 0:
            take = tot[x]
            x += 1
            for i in range(n):
                if take != i and (take, i) not in edge_set and m > 0:
                    m -= 1
                    edge_set.add((take, i))
                    edge_set.add((i, take))
                    edges.append((i, take))
    elif mode == "trianglestar":
        leaves = list(range(1, n))
        #random.shuffle(leaves)
        for i in range(1, n):
            if random.randint(0, 1) == 0:
                edges.append((0, i))
            else:
                edges.append((i, 0))
        j = 0
        while j+2 < n:
            edges.append((leaves[j], leaves[j+1]))
            j += 2
    elif mode == "star12":
        for i in range(2, n):
            edge_set.add((1, i))
            edge_set.add((i, 1))
            edges.append((i, 1))
        while len(edges) < m:
            u = random.randrange(2, n)
            v = random.randrange(2, n)
            while u == v or (u, v) in edge_set:
                u = random.randrange(2, n)
                v = random.randrange(2, n)
            edge_set.add((u, v))
            edge_set.add((v, u))
            edges.append((u, v))
    elif mode == "starpath":
        for i in range(1, n):
            edges.append((i, 0))
        rest = list(range(1, n))
        if n%2 == 1:
            rest = rest[::-1]
        for i in range(m-n+1):
            edges.append((rest[i], rest[i+1]))
    elif mode == "halfpath":
        leaves = min(n-2, (m+1) // 2)
        L = list(range(2, 2+leaves))
        if n%2 == 1:
            L = random.sample(range(2, n), leaves)
        for y in L:
            edges.append((0, y))
        for i in range(leaves-1):
            edges.append((L[i], L[i+1]))
        if n%2 == 1:
            random.shuffle(edges)
    elif mode == "isolated_clique":
        for u in range(1, n):
            for v in range(u+1, n):
                if len(edges) == m:
                    break
                edges.append((u, v))
                


    print(n, len(edges))
    for (u, v) in edges:
        print(u+1, v+1)
    
    
if __name__ == "__main__":
    main()
